#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Tradologics Python SDK
# https://tradologics.com
#
# Copyright 2020-2021 Tradologics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .backtest import Backtest
import requests as _requests

_BASE_URL = "https://api.tradologics.com/v1"
_TOKEN = None
_DEFAULT_TIMEOUT = 5
_SOCKET_URL = "tcp://0.0.0.0:3003"

_IS_BACKTEST = False
_BACKTEST = None


def set_token(token):
    global _TOKEN
    _TOKEN = token


def _set_backtest_mode(start, end):
    global _IS_BACKTEST
    global _BACKTEST
    _IS_BACKTEST = True
    _BACKTEST = Backtest(start, end, _SOCKET_URL)


def _set_current_bar_info(info):
    _BACKTEST.set_current_bar_info(info)


def _get_runtime_events():
    return _BACKTEST.get_runtime_events()


def _process_request(method, url, **kwargs):
    global _IS_BACKTEST

    if "://" in url:
        return _requests.request(method, url, **kwargs)

    if "timeout" not in kwargs:
        kwargs["timeout"] = _DEFAULT_TIMEOUT

    if "headers" not in kwargs:
        if not _TOKEN and not _IS_BACKTEST:
            raise Exception("Please use `set_token(...)` first.")
        kwargs["headers"] = {
            "Authorization": "Bearer {token}".format(token=_TOKEN)
        }

    if _IS_BACKTEST:
        return _BACKTEST.call_eroc_method(method, url, **kwargs)
    else:
        url = "{}/{}".format(_BASE_URL, url.strip('/'))
        return _requests.request(method, url, **kwargs)


def request(method, url, **kwargs):
    return _process_request(method, url, **kwargs)


def get(url, **kwargs):
    return _process_request('GET', url, **kwargs)


def post(url, **kwargs):
    return _process_request('POST', url, **kwargs)


def patch(url, **kwargs):
    return _process_request('PATCH', url, **kwargs)


def put(url, **kwargs):
    return _process_request('PUT', url, **kwargs)


def delete(url, **kwargs):
    return _process_request('DELETE', url, **kwargs)


def options(url, **kwargs):
    return _process_request('OPTIONS', url, **kwargs)


def head(url, **kwargs):
    return _process_request('HEAD', url, **kwargs)


class Session(_requests.Session):
    def _process_request(self, method, url, **kwargs):
        return _process_request(method, url, **kwargs)

    def request(self, method, url, **kwargs):
        return self._process_request(method, url, **kwargs)

    def get(self, url, **kwargs):
        return self._process_request('GET', url, **kwargs)

    def post(self, url, **kwargs):
        return self._process_request('POST', url, **kwargs)

    def patch(self, url, **kwargs):
        return self._process_request('PATCH', url, **kwargs)

    def put(self, url, **kwargs):
        return self._process_request('PUT', url, **kwargs)

    def delete(self, url, **kwargs):
        return self._process_request('DELETE', url, **kwargs)

    def options(self, url, **kwargs):
        return self._process_request('OPTIONS', url, **kwargs)

    def head(self, url, **kwargs):
        return self._process_request('HEAD', url, **kwargs)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Tradologics Python SDK
# https://tradologics.com
#
# Copyright 2020-2021 Tradologics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from requests import Response
import zmq
import json


class Backtest:

    def __init__(self, start, end, socket_url):
        self.start = start
        self.end = end
        self.current_bar_info = {
            'datetime': None,
            'resolution': None,
        }
        context = zmq.Context()
        self._socket = context.socket(zmq.REQ)
        self._socket.connect(socket_url)
        self._runtime_events = {}

    def call_eroc_method(self, method, endpoint, **kwargs):
        request = {
            'method': method,
            'url': endpoint,
            'data': {},
            'headers': {
                'start': self.start,
                'end': self.end,
                'datetime': self.current_bar_info.get('datetime'),
                'resolution': self.current_bar_info.get('resolution')
            }
        }

        if 'json' in kwargs:
            request['data'] = kwargs["json"]

        self._socket.send_json(request)
        res = self._socket.recv_json()
        status = res.get('status')
        self._runtime_events = res.get('events')
        del res['status']
        del res['events']
        response = Response()
        state = {
            '_content': json.dumps(res).encode('utf-8'),
            'status_code': status,
            'encoding': 'utf-8'
        }
        response.__setstate__(state)
        return response

    def set_current_bar_info(self, info):
        self.current_bar_info = info

    def get_runtime_events(self):
        return self._runtime_events



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Tradologics Python SDK
# https://tradologics.com
#
# Copyright 2020-2021 Tradologics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages


def get_version():
    with open("tradologics/__init__.py") as f:
        init = f.read().replace(' ', '')
        return init.split('__version__="')[1].split('"')[0].strip()


def get_requirements():
    with open("requirements.txt") as f:
        return [line.rstrip() for line in f]


def get_description():
    with open("README.rst") as f:
        return f.read()


setup(
    name="tradologics",
    version=get_version(),
    description="Tradologics SDK",
    long_description=get_description(),
    url="https://tradologics.com",
    author="Tradologics, Inc.",
    author_email="opensource@tradologics.com",
    license="Apache 2.0",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 4 - Beta",

        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Office/Business :: Financial",
        "Topic :: Office/Business :: Financial :: Investment",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",

        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    platforms=["any"],
    keywords="tradologics, tradologics.com",
    packages=find_packages(exclude=["contrib", "docs", "tests", "examples"]),
    install_requires=get_requirements()
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Tradologics Python SDK
# https://tradologics.com
#
# Copyright 2020-2021 Tradologics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pandas as _pd


def bars_to_pandas(bars_json, group_by="column"):
    """ convert bars payload to pandas dataframe """
    dfs = newbars = {}

    for dt, values in bars_json['bars'].items():
        for ticker, ohlc in values.items():
            ohlc['ts'] = dt
            if ticker not in newbars:
                newbars[ticker] = []
            newbars[ticker].append(ohlc)

    for ticker, bars in newbars.items():
        dfs[ticker] = _pd.DataFrame(bars)
        dfs[ticker].set_index("ts", inplace=True)
        dfs[ticker].index = _pd.to_datetime(dfs[ticker].index)
        for col in dfs[ticker].columns:
            dfs[ticker][col] = _pd.to_numeric(dfs[ticker][col])

    df = _pd.concat(dfs.values(), axis=1, keys=dfs.keys())
    df.sort_index(level=0, axis=1, inplace=True)

    if group_by == 'column':
        df.columns = df.columns.swaplevel(0, 1)
        df.sort_index(level=0, axis=1, inplace=True)

    return df




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Tradologics Python SDK
# https://tradologics.com
#
# Copyright 2020-2021 Tradologics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from flask import Flask as _flask, Response as _response
from datetime import datetime as _datetime


def start(strategy, endpoint="strategy",
          host="0.0.0.0", port=5000, debug=False):
    a = _FlaskAppWrapper(__name__)
    a.add_endpoint(endpoint, handler=strategy)
    a.run(debug=debug, host=host, port=port)


class _EndpointAction(object):

    def __init__(self, action):
        self.action = action
        self.response = _response(status=200, headers={})

    def __call__(self, *args, **kargs):
        try:
            res = self.action(*args, **kargs)
            return res, 200
        except Exception as err:
            msg = 'Error: {}'.format(err)
            print('{}: {}'.format(str(_datetime.now()).split('.')[0], msg))
            return msg, 500


class _FlaskAppWrapper(object):
    app = None

    def __init__(self, name):
        self.name = name
        self.app = _flask(name)

    def run(self, host, port, debug):
        self.app.run(host, port, debug)

    def add_endpoint(self, endpoint=None, handler=None):
        self.app.add_url_rule(
            endpoint, "strategy", _EndpointAction(handler), methods=["POST"])
